module Livermore
  class CFResponse
    def initialize(response)
      @response = response
    end

    def best_ask
      ask = @response.body['data']['asks'].first

      Order.new(fetch_args(ask).merge(type: Order::ASK))
    end

    def best_bid
      bid = @response.body['data']['bids'].first

      Order.new(fetch_args(bid).merge(type: Order::BID))
    end

    def open_orders
      open_orders = @response.body['data'].select do |order|
        order['status'] =~ /\Aopen|partially_filled\z/
      end

      open_orders.map { |order| Order.new(fetch_args(order)) }
    end

    def code
      @response.code
    end

    def body
      @response.body
    end

    def fetch_args(order)
      {
        market: order['market_name'],
        id: order['id'],
        price: order['price'].to_f,
        amount: order['size'].to_f,
        type: order['order_type']&.to_sym,
        operation: order['operation_type']&.to_sym
      }
    end
  end
end
