module Livermore
  class AbstractStrategy
    TICK_TIME = 0.03

    attr_reader :context, :config

    def initialize(context, config_path)
      @context = context
      @config = YAML.load_file(config_path || default_config_path)
    end

    def trade
      raise 'Abstract method called'
    end

    def method_missing(method, *args)
      if @context.respond_to?(method)
        @context.send(method, *args)
      else
        super
      end
    end

    private

    def default_config_path
      "strategies/#{underscore}.yml"
    end

    def underscore
      self.class.to_s.split('::').last.gsub(/([a-z])([A-Z])/, '\1_\2').downcase
    end

    def info
      raise 'Abstract method called'
    end

    def tick
      print "\033[2J"
      sleep TICK_TIME

      print info.map(&:colorize).join("\n") + "\r\033[#{info.count - 1}A"
      $stdout.flush
    end
  end
end
